import { Injectable } from '@angular/core';
import { Entity, EntityManager, EntityQuery, Predicate, FilterQueryOp } from 'breeze-client';
import { ToasterService, ToasterConfig } from 'angular2-toaster';
import { Moment } from 'moment';
import { RegistrationHelper } from '../model/registration-helper';
import { PurchaseOrder, Shipment, LogEntry, PoStatus } from '../model/entity-model';
import { HttpClient } from '@angular/common/http';
import * as breeze from 'breeze-client';

@Injectable()
export class PoRepositoryService {
  private _em: EntityManager = new EntityManager('http://devapp:8081/breeze/po');
  private apiUrl: string = 'http://devapp:8081/api/owltime/';

  //private _em: EntityManager = new EntityManager('http://localhost:5000/breeze/po');
  //private apiUrl: string = 'http://localhost:5000/owltime/';

  //private _em: EntityManager = new EntityManager('http://localhost:49609/breeze/po');
  //private apiUrl: string = 'http://localhost:49609/owltime/';

  private _initialized: boolean;

  constructor(private toasterService: ToasterService, private http: HttpClient) {
    RegistrationHelper.register(this._em.metadataStore);
    breeze.config.initializeAdapterInstance("uriBuilder", "json");
   }

  public config: ToasterConfig = 
    new ToasterConfig( { positionClass: 'toast-bottom-right' } );
    
  initialize() {
    let promise = new Promise<boolean>((resolve, reject) => {
      if (this._initialized) resolve(true);
      else {
        this._initialized = true;
        this._em.fetchMetadata().then (_ => {
          resolve(true);
        }, error => this.toasterService.pop('error', error));
      }
    })

    return promise;
  }

  popToast(type: string, title: string, body: string) {
    this.toasterService.pop(type, title, body);
  }

  saveChanges() {
    let promise = new Promise((resolve, reject) => {
      this._em.saveChanges().then(
        () => 
        {
          resolve();
          this.toasterService.pop('success', 'Success!', 'Changes Saved Successfully');
        },
        error => 
        {
          reject(error);
          this.toasterService.pop('error', 'Error Saving Changes!', error);
        }
      );
    });

    return promise;
  }

  getApiUrl(): string {
    return this.apiUrl;
  }

  getUserName() {
    return this.http.get(this.apiUrl + 'GetUserName', { responseType: 'text' }); //, { headers: { 'Accept': 'text/plain' } }
  }

  //#region Status

  getStatuses(): Promise<PoStatus[]> {
    let promise = new Promise<PoStatus[]>((resolve, reject) => {
      let query = EntityQuery.from('PoStatuses')
      this._em.executeQuery(query).then(
        queryResult => resolve(<any>queryResult.results),
        error => 
        {
          reject(error);
          this.toasterService.pop('error', 'Error Retrieving Status Records!', error);
        }
      );
    });

    return promise;
  }

  //#endregion Status

  //#region Shipments

  newShipment(): Entity {
    let shipment = this._em.createEntity('Shipment', { shipmentId: 0, shipmentDate: null });
    return shipment;
  }

  newShipmentItem(shipmentId: number, poId: number) : Entity {
    let shipmentItem = this._em.createEntity('ShipmentItem', { shipmentId: shipmentId, purchaseOrderId: poId, quantity: 0 })
    return shipmentItem;
  }

  getShipments(poId: number): Promise<Shipment[]> {
    let promise = new Promise<Shipment[]>((resolve, reject) => {
      let query = EntityQuery.from('Shipments')
      //.expand(['shipmentItems', 'shipmentItems.purchaseOrder'])
      .where('shipmentItems', 'any', 'purchaseOrderId', 'eq', poId);
      this._em.executeQuery(query).then(
        queryResult => resolve(<any>queryResult.results),
        error => 
        {
          reject(error);
          this.toasterService.pop('error', 'Error Retrieving Shipments!', error);
        }
      );
    });

    return promise;
  }

  getShipment(id: number): Promise<Shipment[]> {
    let promise = new Promise<Shipment[]>((resolve, reject) => {
      let query = EntityQuery.from('Shipments').where('shipmentId', 'eq', id).expand(['shipmentItems', 'shipmentItems.purchaseOrder']);
      this._em.executeQuery(query).then(
        queryResult => {
          resolve(<any>queryResult.results)
        },
        error => 
        {
          reject(error);
          this.toasterService.pop('error', 'Error Retrieving Shipment!', error);
        }
      );
    });

    return promise;
  }

  // return po #, part #, description, batchid, shipping sheet #, username
  searchShipments(shippingSheetNumber: string, poNumber: string, partNumber: string, description: string, shipmentDate: Moment): Promise<Shipment[]> {
    let pred: Predicate = Predicate.create('shipmentId', FilterQueryOp.GreaterThan, 0);

    if (poNumber) {
      pred = pred.and(Predicate.create('shipmentItems.purchaseOrder.poNumber', FilterQueryOp.Contains, shippingSheetNumber));
    }

    if (shippingSheetNumber) {
      pred = pred.and(Predicate.create('shippingSheetNumber', FilterQueryOp.Contains, shippingSheetNumber));
    }

    if (partNumber) {
      pred = pred.and(Predicate.create('shipmentItems.purchaseOrder.partNumber', FilterQueryOp.Contains, partNumber));
    }

    if (description) {
      pred = pred.and(Predicate.create('shipmentItems.purchaseOrder.description', FilterQueryOp.Contains, description));
    }

    if (shipmentDate) {
      pred = pred.and(Predicate.create('shipmentDate', FilterQueryOp.Equals, shipmentDate));
    }

    // if (shippedDate) {
    //   pred = pred.and(new Predicate('receivedDate', FilterQueryOp.Equals, shippedDate));
    // }

    let promise = new Promise<Shipment[]>((resolve, reject) => {
      let query = EntityQuery
                  .from('Shipments')
                  .expand(['shipmentItems', 'shipmentItems.purchaseOrder'])
                  .where(pred)
                  .noTracking();

      this._em.executeQuery(query).then(
        queryResult => resolve(<any>queryResult.results),
        error => 
        {
          reject(error);
          this.toasterService.pop('error', 'Error Searching Shipments!', error);
        }
      );
    });

    return promise;
  }

  //#endregion Shipments

  //#region Purchase Orders

  newPurchaseOrder(): Entity {
    let po = this._em.createEntity('PurchaseOrder', { purchaseOrderId: 0, itemId: 10, poStatusId: 1, receivedDate: null, customerPoDate: null, targetDate: null });
    return po;
  }

  getPurchaseOrdersForUser(): Promise<PurchaseOrder[]> {
    let promise = new Promise<PurchaseOrder[]>((resolve, reject) => {
      let query = EntityQuery.from('PurchaseOrdersForUser');
      this._em.executeQuery(query).then(
        queryResult => resolve(<any>queryResult.results),
        error => {
          reject(error);
          this.toasterService.pop('error', 'Error Retrieving Purchase Orders for User!', error);
        }
      );
    });

    return promise;
  }

  getPurchaseOrders(): Promise<PurchaseOrder[]> {
    let promise = new Promise<PurchaseOrder[]>((resolve, reject) => {
      let query = EntityQuery.from('PurchaseOrders');
      this._em.executeQuery(query).then(
        queryResult => resolve(<any>queryResult.results),
        error => 
        {
          reject(error);
          this.toasterService.pop('error', 'Error Retrieving Purchase Orders!', error);
        }
      );
    });

    return promise;
  }

  getPurchaseOrder(id: number): Promise<PurchaseOrder[]> {
    let promise = new Promise<PurchaseOrder[]>((resolve, reject) => {
      let query = EntityQuery.from('PurchaseOrders').where('purchaseOrderId', 'eq', id);
      this._em.executeQuery(query).then(
        queryResult => {
          resolve(<any>queryResult.results)
        },
        error => 
        {
          reject(error);
          this.toasterService.pop('error', 'Error Retrieving Purchase Order!', error);
        }
      );
    });

    return promise;
  }

  searchPurchaseOrders(poNumber: string, partNumber: string, description: string, statusId: number, recvStart: Moment, recvEnd: Moment, shippedDate: Moment): Promise<PurchaseOrder[]> {
    let pred: Predicate = Predicate.create('purchaseOrderId', FilterQueryOp.GreaterThan, 0);

    if (poNumber) {
      pred = pred.and(Predicate.create('poNumber', FilterQueryOp.Contains, poNumber));
    }

    if (partNumber) {
      pred = pred.and(Predicate.create('partNumber', FilterQueryOp.Contains, partNumber));
    }

    if (description) {
      pred = pred.and(Predicate.create('materialDesignation', FilterQueryOp.Contains, description));
    }

    if (statusId) {
      pred = pred.and(Predicate.create('poStatusId', FilterQueryOp.Equals, statusId));
    }

    if (recvStart) {
     pred = pred.and(new Predicate('receivedDate', FilterQueryOp.GreaterThanOrEqual, recvStart));
    }

    if (recvEnd) {
     pred = pred.and(new Predicate('receivedDate', FilterQueryOp.LessThanOrEqual, recvEnd));
    }

    // if (shippedDate) {
    //   pred = pred.and(new Predicate('receivedDate', FilterQueryOp.Equals, shippedDate));
    // }

    let promise = new Promise<PurchaseOrder[]>((resolve, reject) => {
      let query = EntityQuery.from('PurchaseOrders').where(pred);
      this._em.executeQuery(query).then(
        queryResult => resolve(<any>queryResult.results),
        error => 
        {
          reject(error);
          this.toasterService.pop('error', 'Error Searching Purchase Orders!', error);
        }
      );
    });

    return promise;
  }

  //#endregion Purchase Orders

  //#region Documents

  
  getDocuments(poId: number): Promise<Document[]> {
    let promise = new Promise<Document[]>((resolve, reject) => {
      let query = EntityQuery.from('Documents')
                  .where('purchaseOrderId', 'eq', poId)
                  .select('documentId, documentName, documentTypeId, purchaseOrderId, documentType.description');
      this._em.executeQuery(query).then(
        queryResult => resolve(<any>queryResult.results),
        error => 
        {
          reject(error);
          this.toasterService.pop('error', 'Error Retrieving Documents!', error);
        }
      );
    });

    return promise;
  }

  //#endregion Documents

  //#region Log
  getLog(): Promise<LogEntry[]> {
    let promise = new Promise<LogEntry[]>((resolve, reject) => {
      let query = EntityQuery.from('LogEntries');
      this._em.executeQuery(query).then(
        queryResult => resolve(<any>queryResult.results),
        error => 
        {
          reject(error);
          this.toasterService.pop('error', 'Error Retrieving Log Entries!', error);
        }
      );
    });

    return promise;
  }

  //#endregion Log
}
